@@ -3,14 +3,15 @@ package Attribute::Contract;
 use strict;
 use warnings;
 
+use 5.012;
 use attributes;
 
-our $VERSION = '0.02';
-
-use Scalar::Util qw(refaddr);
+our $VERSION = '0.05';
 
 use constant NO_ATTRIBUTE_CONTRACT => $ENV{NO_ATTRIBUTE_CONTRACT};
 
+use Scalar::Util qw(refaddr);
+
 use Attribute::Contract::Modifier::Requires;
 use Attribute::Contract::Modifier::Ensures;
 
@@ -22,10 +23,34 @@ BEGIN {
     @EXPORT = qw(&MODIFY_CODE_ATTRIBUTES &FETCH_CODE_ATTRIBUTES);
 }
 
+our $CONTRACT_REQUIRES_ATTR_ALIAS = 'ContractRequires';
+our $CONTRACT_ENSURES_ATTR_ALIAS  = 'ContractEnsures';
+
 my %attrs;
 my %modifiers;
 my %symcache;
 my %todo;
+my %import;
+
+sub contract_requires_name {
+    $import{-names}->{requires} || $CONTRACT_REQUIRES_ATTR_ALIAS;
+}
+
+sub contract_ensures_name {
+    $import{-names}->{ensures} || $CONTRACT_ENSURES_ATTR_ALIAS;
+}
+
+sub contract_attr_re {
+    my $requires_name = contract_requires_name();
+    my $ensures_name = contract_ensures_name();
+
+    return qr/
+        ^
+        ($requires_name|$ensures_name)
+        (?:\((.*?)\))?
+        $
+    /x;
+}
 
 sub import {
     return if NO_ATTRIBUTE_CONTRACT;
@@ -33,7 +58,10 @@ sub import {
     my ($package) = caller;
     $todo{$package}++;
 
-    __PACKAGE__->export_to_level(1, @_);
+    shift;
+    %import = @_;
+
+    __PACKAGE__->export_to_level(1);
 }
 
 sub CHECK {
@@ -49,7 +77,7 @@ sub CHECK {
             my $attrs = $modifiers{$key};
 
             foreach my $attr (@$attrs) {
-                next unless $attr =~ m/^Contract/;
+                next unless $attr =~ contract_attr_re();
 
                 attributes::->import($package, $code_ref, $attr);
             }
@@ -71,6 +99,9 @@ sub MODIFY_CODE_ATTRIBUTES {
     my $sym = findsym($package, $code_ref);
     my $name = *{$sym}{NAME};
 
+    return if exists $attrs{refaddr $code_ref };
+    return if exists $modifiers{"$package\::$name"};
+
     $attrs{refaddr $code_ref } = \@attr;
     $modifiers{"$package\::$name"} = \@attr;
 
@@ -99,14 +130,16 @@ sub MODIFY_CODE_ATTRIBUTES {
 
     no warnings 'redefine';
     foreach my $attr (@attr) {
-        next unless $attr =~ m/^Contract([^\(]+)(?:\((.*?)\))?/;
+        next unless $attr =~ contract_attr_re();
 
-        my $check     = $1;
+        my $type      = $1;
         my $arguments = $2;
 
-        my $class = __PACKAGE__ . '::Modifier::' . $check;
+        my $modifier = $type eq contract_requires_name() ? 'Requires' : 'Ensures';
 
-        *{$sym} = $class->modify($package, $name, $code_ref, $arguments);
+        my $class = __PACKAGE__ . '::Modifier::' . $modifier;
+
+        *{$sym} = $class->modify($package, $name, $code_ref, \%import, $arguments);
     }
 
     return ();
@@ -142,27 +175,30 @@ Attribute::Contract - Design by contract via Perl attributes
 =head1 SYNOPSIS
 
     package Interface;
-    use AttributeContract;
+    use Attribute::Contract -types => [qw/Str slurpy ArrayRef/];
 
-    sub do_smth :ContractRequires(VALUE, @ANY?) :ContractEnsures(VALUE) {
-        ...;
+    sub do_smth :ContractRequires(Str, slurpy ArrayRef[Str]) :ContractEnsures(Str) {
+        my $self = shift;
+        my ($input_string, $array_ref_of_strings) = @_;
+
+        return '...';
     }
 
     package Implementation;
     use base 'Interface';
-    use AttributeContract;
+    use Attribute::Contract;
 
     sub do_smth {
         my $self = shift;
-        my ($foo, @rest) = @_;
+        my ($input_string, $array_ref_of_strings) = @_;
 
-        return 1;
+        return 'ok';
     }
 
-    Implementaion->do_smth('hi', 'there'); # works
+    Implementation->do_smth('hi', 'there'); # works
 
-    Implementaion->do_smth();              # croaks!
-    Implementaion->do_smth(sub {});        # croaks!
+    Implementation->do_smth();              # croaks!
+    Implementation->do_smth(sub {});        # croaks!
 
 =head1 DESCRIPTION
 
@@ -175,138 +211,69 @@ It's the most useful for interfaces or abstract classes when you want to control
 whether your implementation follows the same interface and respects the Liskov
 substitution principle.
 
-This module does not check the actual types like C<Str>, C<Int> etc, but the
-Perl data types like scalars, arrays, hashes, references and so on. When the type
-does not match a L<Carp>'s C<confess> function will be called with detailed
-information like:
-
-    0 param(s) passed, at least 1 param(s) is required
+This module uses L<Type::Tiny> underneath so all the checks is done via that
+module. Check it out for more documention on type validation.
 
 Why attributes? They feel and look natural and are applied during compile time.
 
-=head2 TYPES
+=head2 IMPORTING
 
-=head3 Scalar types
+When using L<Attribute::Contract> one may want to import various types in order
+to check them in attributes. Types themselves are not imported into the current
+module but rather used when compiling attributes.
 
-=over
+=head3 Types
 
-=item * ANY
+    package MyClass;
+    use Attribute::Contract -types => [qw/ClassName Str/];
 
-Any scalar value is accepted.
+    sub static_method : ContractRequires(ClassName, Str) {
+    };
 
-=item * VALUE
+    ...
 
-Anything but not a reference.
+    MyClass->static_method('string');
 
-=item * REF
+=head3 Type libraries
 
-A non blessed reference to anything.
+When types are complex or the description is too long attributes might get not
+very readable. In this case one can use type libraries (implemented again via
+L<Type::Tiny>):
 
-=item * REF(SCALAR)
+    package MyTypes;
+    use Type::Library -base, -declare => qw(MyInt);
+    use Type::Utils;
+    use Types::Standard qw(Int);
 
-A reference to scalar.
+    declare MyInt, as Int, where => {$_ > 0};
 
-=item * REF(ARRAY)
+    package MyClass;
+    use Attribute::Contract -library => 'MyTypes';
 
-A reference to array.
+    sub static_method : ContractRequires(ClassName, MyInt) {
+    }
 
-=item * REF(HASH)
+    ...
 
-A reference to hash.
+    MyClass->static_method(5);
 
-=item * REF(Regexp)
+=head3 Aliasing
 
-A reference to regular expression.
+If you don't like C<ContractRequires> and C<ContractEnsures> you can set your
+own names:
 
-=item * OBJECT
+    use Attribute::Contract -names => {requires => 'In', ensures => 'Out'}
 
-A blessed reference.
-
-=item * OBJECT(ISA)
-
-A blessed reference with specified isa.
-
-=back
-
-=head3 Greedy types
-
-Types that eat all the elements. Can be specified at the end of the elements
-list for manual unpacking. C<@> stands for arrays and C<%> stands for hashes.
-All the scalar types can be used to specify the types of the elements.
-
-=over
-
-=item * @ARRAY
-
-    @VALUE
-
-Which could mean something like:
-
-    $object->method(1, 2, 3, 4);
-
-=item * %HASH
-
-    %ANY
-
-Which could mean something like:
-
-    $object->method(foo => 'bar', 'baz' => \123);
-
-It also checks that the number of elements is even.
-
-=back
-
-=head2 MULTIPLE VALUES
-
-Use C<,> when specifying several arguments.
-
-    VALUE,ANY,REF(CODE),@VALUE
-
-Which could mean something like:
-
-    $object->method($foo, \@array, sub { ... }, 1, 2, 3);
-
-=head2 ALTERNATIVES
-
-Use C<|> when specifying an alternative type.
-
-    VALUE|REF(VALUE)
-
-Which could mean something like:
-
-    $object->method($foo);
-
-or
-
-    $object->method(\$foo);
-
-Alternatives can be really deep, like this one:
-
-    @(REF(HASH|CODE)|VALUE)
-
-Which is an array of references to hash or code or simple value.
-
-=head2 OPTIONAL VALUES
-
-Use C<?> when specifying an optional value.
-
-    VALUE,VALUE?
-
-Which could mean something like:
-
-    $object->method('foo');
-
-or
-
-    $object->method('foo', 'bar');
+    sub method : In(ClassName, Str) Out(Str) {
+    }
 
 =head2 IMPLEMENTATION
 
 =head3 Inheritance
 
 By default all the contracts are inherited. Just don't forget to C<use>
-L<Attribute::Contract> in the derived class. But if no methods are override then
-even C<using> this module is not needed.
+L<Attribute::Contract> in the derived class. But if no methods are overriden
+then even C<using> this module is not needed.
 
 =head3 Caching
 
@@ -336,7 +303,7 @@ Viacheslav Tykhanovskyi, C<vti@cpan.org>.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2012, Viacheslav Tykhanovskyi
+Copyright (C) 2012-2013, Viacheslav Tykhanovskyi
 
 This program is free software, you can redistribute it and/or modify it under
 the terms of the Artistic License version 2.0.
