@@ -4,23 +4,20 @@ use strict;
 use warnings;
 
 require Carp;
-
-use Attribute::Contract::TypeValidator;
-
-my %cache = ();
+use Attribute::Contract::Utils;
 
 sub modify {
     my $class = shift;
-    my ($package, $name, $code_ref, $attributes) = @_;
+    my ($package, $name, $code_ref, $import, $attributes) = @_;
 
     Carp::croak('Return type(s) are required') unless $attributes;
 
-    my $sub_ref = build($attributes);
+    my $check = build_check(@_);
 
     sub {
         my @return = $code_ref->(@_);
 
-        $sub_ref->(undef, @return);
+        $check->(@return);
 
         @return;
     };
